@@ -16,7 +16,7 @@ extends 'App::Sqitch::Engine';
 sub dbh; # required by DBIEngine;
 with 'App::Sqitch::Role::DBIEngine';
 
-our $VERSION = '0.992';
+our $VERSION = '0.993';
 
 BEGIN {
     # We tell the Oracle connector which encoding to use. The last part of the
@@ -61,6 +61,17 @@ has sqlplus => (
     },
 );
 
+has tmpdir => (
+    is       => 'ro',
+    isa      => 'Path::Class::Dir',
+    required => 1,
+    lazy     => 1,
+    default  => sub {
+        require File::Temp;
+        dir File::Temp::tempdir( CLEANUP => 1 );
+    },
+);
+
 sub key    { 'oracle' }
 sub name   { 'Oracle' }
 sub driver { 'DBD::Oracle 1.23' }
@@ -140,7 +151,7 @@ sub _char2ts {
 
 sub _listagg_format {
     # http://stackoverflow.com/q/16313631/79202
-    return q{COLLECT(%s)};
+    return q{CAST(COLLECT(CAST(%s AS VARCHAR2(512))) AS sqitch_array)};
 }
 
 sub _regex_op { 'REGEXP_LIKE(%s, ?)' }
@@ -196,10 +207,6 @@ sub current_state {
     my $pdtcol = sprintf $self->_ts2char_format, 'c.planned_at';
     my $tagcol = sprintf $self->_listagg_format, 't.tag';
     my $dbh    = $self->dbh;
-    # XXX Oy, placeholders do not work with COLLECT() in this query.
-    # http://www.nntp.perl.org/group/perl.dbi.users/2013/05/msg36581.html
-    # http://stackoverflow.com/q/16407560/79202
-    my $qproj  = $dbh->quote($project // $self->plan->project);
     my $state  = $dbh->selectrow_hashref(qq{
         SELECT * FROM (
             SELECT c.change_id
@@ -215,7 +222,7 @@ sub current_state {
                  , $tagcol AS tags
               FROM changes   c
               LEFT JOIN tags t ON c.change_id = t.change_id
-             WHERE c.project = $qproj
+             WHERE c.project = ?
              GROUP BY c.change_id
                  , c.change
                  , c.project
@@ -228,83 +235,12 @@ sub current_state {
                  , c.planned_at
              ORDER BY c.committed_at DESC
         ) WHERE rownum = 1
-    }) or return undef;
+    }, undef, $project // $self->plan->project) or return undef;
     $state->{committed_at} = _dt $state->{committed_at};
     $state->{planned_at}   = _dt $state->{planned_at};
     return $state;
 }
 
-sub deployed_changes {
-    my $self   = shift;
-    my $tscol  = sprintf $self->_ts2char_format, 'c.planned_at';
-    my $tagcol = sprintf $self->_listagg_format, 't.tag';
-    # XXX Oy, placeholders do not work with COLLECT() in this query.
-    # http://www.nntp.perl.org/group/perl.dbi.users/2013/05/msg36581.html
-    # http://stackoverflow.com/q/16407560/79202
-    my $qproj  = $self->dbh->quote($self->plan->project);
-    return map {
-        $_->{timestamp} = _dt $_->{timestamp};
-        $_;
-    } @{ $self->dbh->selectall_arrayref(qq{
-        SELECT c.change_id AS id, c.change AS name, c.project, c.note,
-               $tscol AS timestamp, c.planner_name, c.planner_email,
-               $tagcol AS tags
-          FROM changes   c
-          LEFT JOIN tags t ON c.change_id = t.change_id
-         WHERE c.project = $qproj
-         GROUP BY c.change_id, c.change, c.project, c.note, c.planned_at,
-               c.planner_name, c.planner_email, c.committed_at
-         ORDER BY c.committed_at ASC
-    }, { Slice => {} } ) };
-}
-
-sub deployed_changes_since {
-    my ( $self, $change ) = @_;
-    my $tscol  = sprintf $self->_ts2char_format, 'c.planned_at';
-    my $tagcol = sprintf $self->_listagg_format, 't.tag';
-    # XXX Oy, placeholders do not work with COLLECT() in this query.
-    # http://www.nntp.perl.org/group/perl.dbi.users/2013/05/msg36581.html
-    # http://stackoverflow.com/q/16407560/79202
-    my $qproj  = $self->dbh->quote($self->plan->project);
-    return map {
-        $_->{timestamp} = _dt $_->{timestamp};
-        $_;
-    } @{ $self->dbh->selectall_arrayref(qq{
-        SELECT c.change_id AS id, c.change AS name, c.project, c.note,
-               $tscol AS timestamp, c.planner_name, c.planner_email,
-               $tagcol AS tags
-          FROM changes   c
-          LEFT JOIN tags t ON c.change_id = t.change_id
-         WHERE c.project = $qproj
-           AND c.committed_at > (SELECT committed_at FROM changes WHERE change_id = ?)
-         GROUP BY c.change_id, c.change, c.project, c.note, c.planned_at,
-               c.planner_name, c.planner_email, c.committed_at
-         ORDER BY c.committed_at ASC
-    }, { Slice => {} }, $change->id) };
-}
-
-sub load_change {
-    my ( $self, $change_id ) = @_;
-    my $tscol  = sprintf $self->_ts2char_format, 'c.planned_at';
-    my $tagcol = sprintf $self->_listagg_format, 't.tag';
-    # XXX Oy, placeholders do not work with COLLECT() in this query.
-    # http://www.nntp.perl.org/group/perl.dbi.users/2013/05/msg36581.html
-    # http://stackoverflow.com/q/16407560/79202
-    my $qcid   = $self->dbh->quote($change_id);
-    my $change = $self->dbh->selectrow_hashref(qq{
-        SELECT c.change_id AS id, c.change AS name, c.project, c.note,
-               $tscol AS timestamp, c.planner_name, c.planner_email,
-                $tagcol AS tags
-          FROM changes   c
-          LEFT JOIN tags t ON c.change_id = t.change_id
-         WHERE c.change_id = $qcid
-         GROUP BY c.change_id, c.change, c.project, c.note, c.planned_at,
-               c.planner_name, c.planner_email
-    }, undef) || return undef;
-    $change->{timestamp} = _dt $change->{timestamp};
-    return $change;
-}
-
 sub is_deployed_change {
     my ( $self, $change ) = @_;
     $self->dbh->selectcol_arrayref(
@@ -585,15 +521,61 @@ sub begin_work {
     return $self;
 }
 
+sub _file_for_script {
+    my ($self, $file) = @_;
+
+    # Just use the file if no special character.
+    if ($file !~ /[@?%\$]/) {
+        $file =~ s/"/""/g;
+        return $file;
+    }
+
+    # Alias or copy the file to a temporary directory that's removed on exit.
+    (my $alias = $file->basename) =~ s/[@?%\$]/_/g;
+    $alias = $self->tmpdir->file($alias);
+
+    # Remove existing file.
+    if (-e $alias) {
+        $alias->remove or hurl oracle => __x(
+            'Cannot remove {file}: {error}',
+            file  => $alias,
+            error => $!
+        );
+    }
+
+    if ($^O eq 'MSWin32') {
+        # Copy it.
+        $file->copy_to($alias) or hurl oracle => __x(
+            'Cannot copy {file} to {alias}: {error}',
+            file  => $file,
+            alias => $alias,
+            error => $!
+        );
+    } else {
+        # Symlink it.
+        $alias->remove;
+        symlink $file->absolute, $alias or hurl oracle => __x(
+            'Cannot symlink {file} to {alias}: {error}',
+            file  => $file,
+            alias => $alias,
+            error => $!
+        );
+    }
+
+    # Return the alias.
+    $alias =~ s/"/""/g;
+    return $alias;
+}
+
 sub run_file {
     my $self = shift;
-    (my $file = shift) =~ s/"/""/g;
+    my $file = $self->_file_for_script(shift);
     $self->_run(qq{\@"$file"});
 }
 
 sub run_verify {
     my $self = shift;
-    (my $file = shift) =~ s/"/""/g;
+    my $file = $self->_file_for_script(shift);
     # Suppress STDOUT unless we want extra verbosity.
     my $meth = $self->can($self->sqitch->verbosity > 1 ? '_run' : '_capture');
     $self->$meth(qq{\@"$file"});
@@ -624,15 +606,16 @@ sub log_revert_change {
     $sth->execute;
 
     # Retrieve dependencies.
-    my ($req, $conf) = $dbh->selectrow_array(q{
+    my $depcol = sprintf $self->_listagg_format, 'dependency';
+    my ($req, $conf) = $dbh->selectrow_array(qq{
         SELECT (
-            SELECT COLLECT(dependency)
+            SELECT $depcol
               FROM dependencies
              WHERE change_id = ?
                AND type = 'require'
         ),
         (
-            SELECT COLLECT(dependency)
+            SELECT $depcol
               FROM dependencies
              WHERE change_id = ?
                AND type = 'conflict'
